import re

from panflute import Doc, Element, MetaMap, RawInline, Str

from .data import (
    AcronymEntry,
    GlossaryEntry,
    get_acronym_entry,
    get_glossary_entry,
    set_acronym_entry,
    set_glossary_entry,
)

filter_cmd_re = re.compile(r"{(\w+):(\w+)}")
glossary_cmds = {
    "g": "gls",
    "gs": "glspl",
    "G": "Gls",
    "Gs": "Glspl",
}
acronym_cmds = {
    "a": "gls",
}


def glossary(elem: Element, doc: Doc):
    """Pandoc filter that does all the magic :D

    Args:
        key (str): Pandoc element type
        value (str): Pandoc element content
        format_ (str): Output file format
        meta (dict): File metadata

    Returns:
        _type_: Pandoc element iff the current element is a glossary entry or an acronym
    """
    if doc.format not in {"latex", "json"} or not isinstance(elem, Str):
        return None

    value: str = elem.text
    match = filter_cmd_re.search(value)
    if match is None:
        return None
    cmd: str = match.groups()[0]
    is_acronym = cmd in acronym_cmds
    is_glossary = cmd in glossary_cmds
    if not is_acronym and not is_glossary:
        return None
    cmd = (glossary_cmds if is_glossary else acronym_cmds)[cmd]
    label: str = match.groups()[1]

    doc.metadata["has-glossary"] = True
    if is_glossary:
        if "glossary-entries" not in doc.metadata:
            doc.metadata["glossary-entries"] = {}
        entry = get_glossary_entry(label)
        if entry is None:
            entry = GlossaryEntry("", "")
            set_glossary_entry(label, entry)

        glossary_entries: MetaMap = doc.metadata["glossary-entries"]  # type: ignore
        if label not in glossary_entries:
            entry = get_glossary_entry(label)
            assert entry is not None
            latex_entry = {
                "name": entry.name,
                "description": entry.description,
            }
            if entry.plural:
                latex_entry["plural"] = entry.plural
            glossary_entries[label] = latex_entry
    else:
        if "acronym-entries" not in doc.metadata:
            doc.metadata["acronym-entries"] = {}
        entry = get_acronym_entry(label)
        if entry is None:
            entry = AcronymEntry("", "")
            set_acronym_entry(label, entry)
        acronym_entries: MetaMap = doc.metadata["acronym-entries"]  # type: ignore
        if label not in acronym_entries:
            acronym_entries[label] = {
                "name": entry.name,
                "description": entry.description,
            }

    return RawInline(
        filter_cmd_re.sub(f"\\\\{cmd}{{{label}}}" + ("{$_A$}" if is_acronym else "{$_G$}"), value), "latex"
    )



import pathlib
from dataclasses import dataclass, field
from typing import Iterable, Optional

import yaml
from loguru import logger


@dataclass
class GlossaryEntry:
    """
    A class representing a glossary entry.

    Attributes:
        name: Word(s) that should be rendered in occurences of the glossary entry
        description: Explanation of the word's meaning
        plural: Eventual word plural form
    """

    name: str
    description: str
    plural: Optional[str] = None


@dataclass
class AcronymEntry:
    """
    A class representing an acronym.

    Attributes:
        name: Acronym that should be rendered
        description: Acronym meaning
    """

    name: str
    description: str


@dataclass
class Data:
    """
    Class used for internal state storage
    """

    glossary_data: dict[str, GlossaryEntry] = field(default_factory=dict)
    glossary_dirty: bool = False
    acronym_data: dict[str, AcronymEntry] = field(default_factory=dict)
    acronym_dirty: bool = False


__DATA = Data()


def load_file(file_name: str):
    """Loads a .yaml file, defaulting to an empty dict if the file is not found.

    Args:
        file_name (str): name of the file to be loaded

    Returns:
        Any: loaded dict
    """
    try:
        return yaml.load(pathlib.Path(file_name).read_text(encoding="utf-8"), yaml.Loader)
    except FileNotFoundError:
        logger.warning(f"{file_name} not found, using empty one")
        return {}


def save_file(file_name: str, data):
    """Saves a .yaml file

    Args:
        file_name (str): name of the file to be loaded
        data (Any): data to be saved
    """
    pathlib.Path(file_name).write_text(yaml.dump(data, default_flow_style=False), encoding="utf-8")


def load_data():
    """Loads filter data"""
    __DATA.glossary_data = load_file("glossary.yaml")
    __DATA.acronym_data = load_file("acronyms.yaml")


def save_data():
    """Saves filter data"""
    if __DATA.glossary_dirty:
        save_file("glossary.yaml", __DATA.glossary_data)
    if __DATA.acronym_dirty:
        save_file("acronyms.yaml", __DATA.acronym_data)


def set_glossary_entry(label: str, entry: GlossaryEntry):
    """Sets a glossary entry for a label

    Args:
        label (str): entry label
        entry (GlossaryEntry): glossary entry
    """
    __DATA.glossary_dirty = True
    __DATA.glossary_data[label] = entry


def get_glossary_entry(label: str) -> Optional[GlossaryEntry]:
    """Gets a glossary entry from a label

    Args:
        label (str): entry label

    Returns:
        Optional[GlossaryEntry]: glossary entry if found, `None` otherwise
    """
    return __DATA.glossary_data.get(label, None)


def get_glossary_entries() -> Iterable[tuple[str, GlossaryEntry]]:
    """Gets all the registered glossary entries

    Returns:
        Iterable[tuple[str, GlossaryEntry]]: entries
    """
    return __DATA.glossary_data.items()


def set_acronym_entry(label: str, entry: AcronymEntry):
    """Sets an acronym entry for a label

    Args:
        label (str): entry label
        entry (AcronymEntry): acronym entry
    """
    __DATA.acronym_dirty = True
    __DATA.acronym_data[label] = entry


def get_acronym_entry(label: str) -> Optional[AcronymEntry]:
    """Gets an acronym entry from a label

    Args:
        label (str): entry label

    Returns:
        Optional[AcronymEntry]: acronym entry if found, `None` otherwise
    """
    return __DATA.acronym_data.get(label, None)


def get_acronym_entries() -> Iterable[tuple[str, AcronymEntry]]:
    """Gets all the registered acronym entries

    Returns:
        Iterable[tuple[str, AcronymEntry]]: entries
    """
    return __DATA.acronym_data.items()



import sys

from loguru import logger
from panflute import toJSONFilter

from pandoc_glossary_filter.filter import glossary

from .data import get_acronym_entries, get_glossary_entries, load_data, save_data


def main():
    """Main"""
    load_data()
    toJSONFilter(glossary)
    save_data()
    incomplete_data: bool = False
    for label, entry in get_glossary_entries():
        if not entry.description:
            incomplete_data = True
            logger.error(f"Empty description for glossary label {label}")
        if not entry.name:
            incomplete_data = True
            logger.error(f"Empty name for glossary label {label}")
    for label, entry in get_acronym_entries():
        if not entry.description:
            incomplete_data = True
            logger.error(f"Empty description for acronym label {label}")
        if not entry.name:
            incomplete_data = True
            logger.error(f"Empty name for acronym label {label}")
    if incomplete_data:
        sys.exit(1)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pandoc_glossary_filter']

package_data = \
{'': ['*']}

install_requires = \
['loguru>=0.6.0,<0.7.0', 'panflute>=2.2.3,<3.0.0', 'pyyaml>=6.0,<7.0']

entry_points = \
{'console_scripts': ['pandoc-glossary = pandoc_glossary_filter:main']}

setup_kwargs = {
    'name': 'pandoc-glossary-filter',
    'version': '0.2.0',
    'description': 'Pandoc filter used to generate a glossary and an acronym list common to a set of documents',
    'long_description': '# pandoc_glossary_filter\n\n## Usage\n\n```md\n<!-- GLOSSARY -->\n{g:mylabel} <!-- Inserts a glossary entry with label `mylabel` -->\n{G:mylabel} <!-- Inserts a glossary entry with label `mylabel` capitalizing the first letter -->\n{gs:mylabel} <!-- Inserts a glossary entry with label `mylabel` using the plural form -->\n{Gs:mylabel} <!-- Inserts a glossary entry with label `mylabel` capitalizing the first letter and using the plural form -->\n\n<!-- ACRONYMS -->\n{a:myacro} <!-- Insert an acronym with label `myacro` -->\n```\n\n## TODO\n\n- Add support for a full glossary document containing all the defined entries\n- Actually usable documentation\n',
    'author': 'Augusto Zanellato',
    'author_email': 'augusto.zanellato@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7.2,<4.0.0',
}


setup(**setup_kwargs)
